#! /usr/local/bin/perl -w

# File: findClosestDefn.pl
# Author: Mark Stevenson
# Date started: 7/7/2008
# Last updated: 8/7/2008
# Purpose: Reads a list of CUIs and prints out the definitions (from MRDEF file) 
#          for that CUI. If the CUI doesn't have a defintion then follows 
#          related CUIs (from MRREL file) until one with a defintion is found. 
#          The semantic type(s) for each CUI is also printed


# Read definitions
print STDERR "Reading definitions\t\t";
%def = ();
$def_file = "/export/nlp-humber/UMLS/UMLS_2007AB/subsets/cat3/2007AB/META/MRDEF";
open(DEF, $def_file) or die "Can't open $def_file for reading\n";
while(<DEF>) {
  if(/^(C[0-9]+)\|(.+)/) {
    if(exists $def{$1}) {
      $def{$1} .= "\n";
    } # if
    $def{$1} .= $2;
  } # if
} # while
close(DEF);
print STDERR "done\n";


# Read semantic types
print STDERR "Reading semantic types\t\t";
%sty = ();
$mrsty_file = "/export/nlp-humber/UMLS/UMLS_2007AB/subsets/cat3/2007AB/META/MRSTY";
open(STY, $mrsty_file) or die "Can't open $mrsty_file for reading\n";
while(<STY>) {
  if(/^(C[0-9]+)\|[^\|]+\|(.+)\|/) {
     if(exists $sty{$1}) {
      $sty{$1} .= "|";
    } # if
    $sty{$1} .= $2;
  } # if
} # while
close(STY);
print STDERR "done\n";



# Read through list of CUI relations
print STDERR "Reading concept relations\t";
%relations = ();     # $relations{$cui1}{$cui2} = $relns
$concept_rels_file = "/export/nlp-humber/UMLS/UMLS_2007AB/subsets/cat3/2007AB/META/MRREL";
open(RELS, $concept_rels_file) or die "Can't open $concept_rels_file for reading\n";
while(<RELS>) {
  # Read through list of related CUIs
  if(/^(C[0-9]+)\|([A-Z]+)\|(C[0-9]+)/) {
    $cui1 = $1;
    $reln = $2;
    $cui2 = $3;

    # Control which relation types to store
    if($reln =~/^(RB|RN|RO|PAR|CHD|SIB|RL|AQ|QB)$/) {
      $relations{$cui1}{$cui2} = $reln;
    } #if
  } #if
} # while
close(RELS);
print STDERR "done\n";



# Main part of code
# Prints out definitions for each CUI in command line arguments
foreach $argnum (0 .. $#ARGV) {
  $cui = $ARGV[$argnum];

  print "Looking for definition for CUI $cui\n";
  if(exists $sty{$cui}) {
    print "Semantic type: $sty{$cui}\n";
  } # if

  %paths = ();
  $paths{$cui} = $cui;
  
  
  &getDef(\%paths);
  print "\n";

} # foreach 





###############################################################################
#                             SUBROUTINES
###############################################################################



# &getDef($paths_ref)
#
# Purpose: Reads through a hash which containing paths through UMLS (the format
#          of this hash is that the keys are paths and the values the last CUI
#          in the path). Checks wheteher there is a defintion for the last 
#          CUI in the path -- stops if one is found, otherwise finds next CUI(s)
#          in path and searches again
 
sub getDef {
  
  my ($paths_ref) = @_;

  # Check each of the paths to see whether it has a defn
  $defns_found = 0;
  foreach $path (keys %$paths_ref) {
    if(exists $def{$$paths_ref{$path}}) {
      print "$path\n$def{$$paths_ref{$path}}\n";
      $defns_found++;
    } # if
  } # foreach 

  # Stop if at least one definition was found, otherwise extend path and search
  # again
  if($defns_found > 0) {
    return $defns_found;
  } else {
    # Add to each path 
    %extended_paths = ();
    foreach $path (keys %$paths_ref) {
      $last_cui = $$paths_ref{$path};
      foreach $related_cui (keys %{$relations{$last_cui}}) {
	$extended_path = $path;
	# Check $related_cui hasn't already been seen, if not then add it 
	# to the path
	if($path !~ /$related_cui/) {
	  $extended_path .= "-".$relations{$last_cui}{$related_cui}."-".$related_cui;
	  $extended_paths{$extended_path} = $related_cui;
	} # unless
      } # foreach 
      
      &getDef(\%extended_paths);
    } # if/else
  } # if/else
} # sub getDef
  
